/*
   ZoranI2C.c

   ZORAN ZR36120 module driver.
   Copyright (C) 1997  Vincent Labie (try ivsparis@pratique.fr with
				      for Vincent Labie in the suject)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"ZoranI2C.h"
#include	"ZoranIO.h"

#define		I2CTIMEOUT	10
#define		I2CADRRETRY	10

/**********************************************************************************************************************/
static	BOOL	I2CGetSDA(const ZORAN * pZoran)
{
	ZR36120 *		pZR		=(ZR36120*)pZoran->pZR36120;
	BOOL			bState;
	udelay(1);
	if (pZR->I2CControl.SDA)	bState=TRUE;
	else				bState=FALSE;
	I2CDBG2(("  I2CGetSDA got %i\n",bState));
	return(bState);
}

/**********************************************************************************************************************/
static	BOOL	I2CGetSCL(const ZORAN * pZoran)
{
	ZR36120 *		pZR		=(ZR36120*)pZoran->pZR36120;
	BOOL			bState;
	udelay(1);
	if (pZR->I2CControl.SCL)	bState=TRUE;
	else				bState=FALSE;
	I2CDBG2(("  I2CGetSCL got %i\n",bState));
	return(bState);
}

/**********************************************************************************************************************/
static	void	I2CReleaseSDA(const ZORAN * pZoran)
{
	ZR36120 *		pZR		=(ZR36120*)pZoran->pZR36120;
	pZR->I2CControl.SDA=1;
	udelay(1);
	I2CDBG2(("I2CReleaseSDA\n"));
}

/**********************************************************************************************************************/
static	BOOL	I2CSetSDA(const ZORAN * pZoran,BOOL bState)
{
	ZR36120 *		pZR		=(ZR36120*)pZoran->pZR36120;
	int			TimeOut;
	BOOL			bStill;
	bState=(bState!=0);
	pZR->I2CControl.SDA=bState;
	for (bStill=TRUE,TimeOut=I2CTIMEOUT;(TimeOut>0)&&(bStill);TimeOut--)
		{ udelay(1); if (I2CGetSDA(pZoran)==bState) bStill=FALSE; }
	udelay(1);
	I2CDBG2((" I2CSetSDA %i => ",bState)); I2CDBG2(((bStill)?"ERROR\n":"OK\n"));
	if (bStill) I2CDBG2(("I2CSetSDA %i FAILED\n",bState));
	return(bStill==FALSE);
}

/**********************************************************************************************************************/
static	BOOL	I2CSetSCL(const ZORAN * pZoran,BOOL bState)
{
	ZR36120 *		pZR		=(ZR36120*)pZoran->pZR36120;
	int			TimeOut;
	BOOL			bStill;
	bState=(bState!=0);
	pZR->I2CControl.SCL=bState;
	for (bStill=TRUE,TimeOut=I2CTIMEOUT;(TimeOut>0)&&(bStill);TimeOut--)
		{ udelay(1); if (I2CGetSCL(pZoran)==bState) bStill=FALSE; }
	udelay(1);
	I2CDBG2((" I2CSetSCL %i => ",bState)); I2CDBG2(((bStill)?"ERROR\n":"OK\n"));
	if (bStill) I2CWAR(("I2CSetSCL %i FAILED\n",bState));
	return(bStill==FALSE);
}

/**********************************************************************************************************************/
static	BOOL	I2CSendStart(const ZORAN * pZoran)
{
	/* SCL and SDA must be high */
	I2CDBG(("I2CSendStart\n"));
	if ((I2CGetSDA(pZoran)==0)||(I2CGetSCL(pZoran)==0))
		{ I2CWAR(("I2CSendStart wong lines states\n")); return(FALSE); }
	I2CSetSDA(pZoran,0);
	udelay(1);
	I2CSetSCL(pZoran,0);
	udelay(1);
	return(TRUE);
}


/**********************************************************************************************************************/
static	BOOL	I2CSendStop(const ZORAN * pZoran)
{
	/* SCL must be low */
	I2CDBG(("I2CSendStop\n"));
	if (I2CGetSCL(pZoran)!=0)
		{ I2CWAR(("I2CSendStop wong lines states\n"));
		if (I2CSetSCL(pZoran,0)==FALSE)
			{ I2CWAR(("I2CSendStop can't set scl low\n")); return(FALSE); }
		}
	I2CSetSDA(pZoran,0);
	I2CSetSCL(pZoran,1); 
	udelay(1);
	I2CSetSDA(pZoran,1);
	udelay(1);
	return(TRUE);
}


/**********************************************************************************************************************/
static	BOOL	I2CSendByte(const ZORAN * pZoran,u8 Byte)
{
	int			Mask;
	BOOL			bError;

	/* SCL must be low, no start/stop sequence */
	I2CDBG(("I2CSendByte 0x%02X,%c\n",(int)Byte,(Byte!=0)?Byte:' '));
	for (Mask=0x80;Mask!=0;Mask>>=1)
		{
		I2CSetSDA(pZoran,Byte&Mask);
		if (I2CSetSCL(pZoran,1)==FALSE)
			{ I2CWAR(("I2CSendByte SCL timeout error\n"));
			I2CSetSDA(pZoran,1); return(FALSE); }
		udelay(1);
		if (((Byte&Mask)!=0)&&(I2CGetSDA(pZoran)==0))
			{ I2CWAR(("I2CSendByte SDA conflict error with mask %i\n",(int)Mask)); return(FALSE); }
		I2CSetSCL(pZoran,0);
		}
	I2CDBG(("I2CSendByte get ack bit\n"));
	I2CReleaseSDA(pZoran);
	if (I2CSetSCL(pZoran,1)==FALSE) { I2CWAR(("I2CSendByte SCL ack timeout\n")); return(FALSE); }
	udelay(1);
	bError=I2CGetSDA(pZoran);
	I2CDBG(("I2CSendByte ack=%i\n",bError)); if (bError) I2CWAR(("I2CSendByte no ack error\n"));
	I2CSetSCL(pZoran,0);
	udelay(1);
	return(bError==0);
}

/**********************************************************************************************************************/
static	int	I2CGetByte(const ZORAN * pZoran)
{
	int			Mask;
	u8			Byte;

	/* SCL must be low, no start/stop sequence */
	for (Byte=0,Mask=0x80;Mask!=0;Mask>>=1)
		{
		I2CSetSDA(pZoran,1);
		if (I2CSetSCL(pZoran,1)==FALSE)
			{ I2CWAR(("I2CGetByte SCL timeout error\n"));
			I2CSetSDA(pZoran,1); return(-1); }
		if (I2CGetSDA(pZoran)) Byte+=Mask;
		I2CSetSCL(pZoran,0);
		I2CSetSDA(pZoran,0);
		}
	I2CDBG(("I2CGetByte %i\n",Byte));
	return(Byte);
}


/**********************************************************************************************************************/
	int	I2CWrite(const ZORAN * pZoran,int I2CAdr,int I2CSubAdr,const char * TabByte,int NbMaxByte)
{
	int		IndexByte;
	int		Retry;

	for(Retry=0;TRUE;Retry++)
		{
		if (I2CSendStart(pZoran))
			if (I2CSendByte(pZoran,I2CAdr))
				{
				I2CDBG(("I2CWrite ADR 0x%02X ok\n",I2CAdr));
				if (I2CSubAdr!=-1)
					{ if (I2CSendByte(pZoran,I2CSubAdr)) 
						{ I2CDBG(("I2CWrite SubADR 0x%02X ok\n",I2CSubAdr)); break; }
					}
				else	break; 
				}
		I2CSendStop(pZoran);
		if (Retry>=I2CADRRETRY) { I2CWAR(("I2CWrite ADR 0x%02X timeout error\n",I2CAdr)); return(-1); }
		}
	for(IndexByte=0;IndexByte<NbMaxByte;IndexByte++)
		if (I2CSendByte(pZoran,TabByte[IndexByte])==FALSE)
			{ I2CWAR(("I2CWrite error after %i bytes\n",IndexByte));
			I2CSendStop(pZoran); return(-1); }
		else	I2CDBG(("I2CWrite byte nb %i OK\n",IndexByte));
	I2CSendStop(pZoran);
	I2CDBG(("I2CWrite %i bytes OK\n",IndexByte));
	return(IndexByte);
}

/**********************************************************************************************************************/
	int	I2CRead(const ZORAN * pZoran,int I2CAdr,int I2CSubAdr,char * TabByte,int NbMaxByte)
{
	int		IndexByte;
	int		Retry;

	for(Retry=0;TRUE;Retry++)
		{
		if (I2CSendStart(pZoran))
			if (I2CSendByte(pZoran,I2CAdr))
				{
				I2CWAR(("I2CRead ADR 0x%02X ok\n",I2CAdr));
				if (I2CSubAdr!=-1)
					{ if (I2CSendByte(pZoran,I2CSubAdr))
						{ I2CWAR(("I2CRead SubADR 0x%02X ok\n",I2CSubAdr)); break; }
					}
				else	break;
				}
		I2CSendStop(pZoran);
		if (Retry>=I2CADRRETRY) { I2CWAR(("I2CRead ADR 0x%02X timeout error\n",I2CAdr)); return(-1); }
		}
	for(IndexByte=0;IndexByte<NbMaxByte;IndexByte++)
		{
		TabByte[IndexByte]=I2CGetByte(pZoran);
		if (IndexByte==NbMaxByte-1)	I2CSetSDA(pZoran,1);
		else				I2CSetSDA(pZoran,0);
		if (I2CSetSCL(pZoran,1)==FALSE)
			{ I2CWAR(("I2CRead SCL timeout after %i bytes\n",IndexByte));
			I2CSetSDA(pZoran,1); return(-1); }
		else	I2CDBG(("I2CRead byte nb %i OK\n",IndexByte));
		I2CSetSCL(pZoran,0);// I2CSetSCL(pZoran,1);
		}
	I2CSendStop(pZoran);
	I2CDBG(("I2CRead %i bytes OK\n",IndexByte));
	return(IndexByte);
}

